#include-once
;

Global Const $WH_MOUSE_LL 					= 14
Global Const $MOUSE_PRIMARYDOWN_EVENT		= 513
Global Const $MOUSE_PRIMARYUP_EVENT			= 514

Global $hCtrlEvent_CallBack 				= 0
Global $hCtrlEvent_MHook 					= 0

Global $aCtrlEvents[1][1]

; #FUNCTION# ====================================================================================================================
; Name...........: _WinControlSetEvent
; Description ...: Sets an event for Window control.
; Syntax.........: Func(hWindow, nCtrlID, $sCallOnHoldFunc [, $sOnHoldParams [, $sCallOnReleaseFunc [, $sOnReleaseParams]]])
;
; Parameters ....: $hWindow            - Parent window handle (or class name) that have the control to set event for.
;                  $nCtrlID            - CtrlID (or control text) of the control to set the event for.
;                  $sCallOnHoldFunc    - Function name to call when event is fired (control is pressed *down*).
;                  $sOnHoldParams      - [Optional] Extra parameters to pass to the called function ($sCallOnHoldFunc).
;                  $sCallOnReleaseFunc - [Optional] Function to call when event is fired (control is released after pressing down).
;                  $sOnReleaseParams   - [Optional] Extra parameters to pass to the called function ($sCallOnReleaseFunc).
;
; Return values .: Always return 1.
;
; Author ........: G.Sandler (a.k.a MrCreatoR).
; Modified.......: 22.01.2009, 20:00
;
; Remarks .......: 1) This UDF includes OnAutoItExit function to release the callback/hooks resources.
;                  2) Blocking of $sCallOnHoldFunc/$sCallOnReleaseFunc function by window messages with commands
;                     such as "Msgbox()" can lead to unexpected behavior, the return to the system should be as fast as possible!
;
; Related .......: _WinCtrlEvents_Handler, _ControlGetHoveredID, _WinGetHoveredHandle, _OnAutoItExit
; Link ..........; 
; Example .......; Yes
; ===============================================================================================================================
Func _WinControlSetEvent($hWindow, $nCtrlID, $sCallOnHoldFunc, $sOnHoldParams="", $sCallOnReleaseFunc="", $sOnReleaseParams="")
	If $aCtrlEvents[0][0] = 0 Then
		$hCtrlEvent_CallBack = DllCallbackRegister("_WinCtrlEvents_Handler", "int", "int;ptr;ptr")
		
		Local $hM_Module = DllCall("Kernel32.dll", "hwnd", "GetModuleHandle", "ptr", 0)
		Local $hM_Hook = DllCall("User32.dll", "hwnd", "SetWindowsHookEx", "int", $WH_MOUSE_LL, _
			"ptr", DllCallbackGetPtr($hCtrlEvent_CallBack), "hwnd", $hM_Module[0], "dword", 0)
		
		$hCtrlEvent_MHook = $hM_Hook[0]
	EndIf
	
	$aCtrlEvents[0][0] += 1
	ReDim $aCtrlEvents[$aCtrlEvents[0][0]+1][6]
	
	$aCtrlEvents[$aCtrlEvents[0][0]][0] = $hWindow
	$aCtrlEvents[$aCtrlEvents[0][0]][1] = $nCtrlID
	$aCtrlEvents[$aCtrlEvents[0][0]][2] = $sCallOnHoldFunc
	$aCtrlEvents[$aCtrlEvents[0][0]][3] = $sOnHoldParams
	$aCtrlEvents[$aCtrlEvents[0][0]][4] = $sCallOnReleaseFunc
	$aCtrlEvents[$aCtrlEvents[0][0]][5] = $sOnReleaseParams
	
	Return 1
EndFunc

Func _WinCtrlEvents_Handler($nCode, $wParam, $lParam)
	Local $iEvent = BitAND($wParam, 0xFFFF)
	
	If $iEvent = $MOUSE_PRIMARYUP_EVENT Or $iEvent = $MOUSE_PRIMARYDOWN_EVENT Then
		Local $hWindow = _WinGetHoveredHandle()
		Local $aCtrlID = _ControlGetHoveredID()
		
		Local $sCtrlData = ControlGetText($hWindow, "", $aCtrlID[1])
		
		For $i = 1 To $aCtrlEvents[0][0]
			If ($aCtrlEvents[$i][0] = $hWindow Or WinGetHandle($aCtrlEvents[$i][0]) = $hWindow) And ($aCtrlID[1] <> 0 And _
				($aCtrlEvents[$i][1] = $aCtrlID[1] Or $aCtrlEvents[$i][1] = $sCtrlData Or _
					$aCtrlEvents[$i][1] = _WinGetClassNameNN($aCtrlID[1], $hWindow))) Then
				
				If $iEvent = $MOUSE_PRIMARYDOWN_EVENT Then
					Call($aCtrlEvents[$i][2], $aCtrlEvents[$i][3], $hWindow, $aCtrlID[1])
					If @error Then Call($aCtrlEvents[$i][2], $aCtrlEvents[$i][3], $hWindow)
					If @error Then Call($aCtrlEvents[$i][2], $aCtrlEvents[$i][3])
					If @error Then Call($aCtrlEvents[$i][2])
				ElseIf $iEvent = $MOUSE_PRIMARYUP_EVENT Then
					Call($aCtrlEvents[$i][4], $aCtrlEvents[$i][5], $hWindow, $aCtrlID[1])
					If @error Then Call($aCtrlEvents[$i][4], $aCtrlEvents[$i][5], $hWindow)
					If @error Then Call($aCtrlEvents[$i][4], $aCtrlEvents[$i][5])
					If @error Then Call($aCtrlEvents[$i][4])
				EndIf
				
				ExitLoop
			EndIf
		Next
	EndIf
EndFunc

Func _ControlGetHoveredID()
	Local $aRet[2]
	Local $iOld_Opt_MCM = Opt("MouseCoordMode", 1)
	
	Local $hRet = DllCall("user32.dll", "int", "WindowFromPoint", "long", MouseGetPos(0), "long", MouseGetPos(1))
	
	$aRet[0] = $hRet[0]
	
	$hRet = DllCall("user32.dll", "int", "GetDlgCtrlID", "hwnd", $hRet[0])
	If $hRet[0] < 0 Then $hRet[0] = 0
	
	Opt("MouseCoordMode", $iOld_Opt_MCM)
	
	$aRet[1] = $hRet[0]
	
	Return $aRet
EndFunc

Func _WinGetHoveredHandle()
	Local $iOld_Opt_MCM = Opt("MouseCoordMode", 1)
	Local $aRet = DllCall("user32.dll", "int", "WindowFromPoint", "long", MouseGetPos(0), "long", MouseGetPos(1))
	
	Opt("MouseCoordMode", $iOld_Opt_MCM)
	
	$aRet = DllCall("User32.dll", "hwnd", "GetAncestor", "hwnd", $aRet[0], "uint", 2) ;$GA_ROOT
	
	Return HWnd($aRet[0])
EndFunc

Func _WinGetClassNameNN($hCtrlWnd, $hWinWnd="")
	$hCtrlWnd = ControlGetHandle($hWinWnd, "", $hCtrlWnd)
	$sClassName = _WinGetClassName($hCtrlWnd)
	$aClassList = StringSplit(WinGetClassList($hWinWnd), @CRLF)
    
	For $iC In $aClassList
		If $iC = $sClassName Then
			For $j = 1 To UBound($aClassList)
				If ControlGetHandle($hWinWnd, "", $iC & $j) = $hCtrlWnd Then Return $iC & $j
			Next
		EndIf
	Next
	
	Return $sClassName
EndFunc

Func _WinGetClassName($hWnd)
	Local $aClassName = DLLCall("user32.dll", "int", "GetClassName", "hWnd", $hWnd, "str", "", "int", 256)
	
	If Not @error And $aClassName[0] <> 0 Then Return $aClassName[2]
	Return @error
EndFunc

Func _OnAutoItExit()
	If IsArray($hCtrlEvent_MHook) And $hCtrlEvent_MHook[0] > 0 Then
		DllCall("User32.dll", "int", "UnhookWindowsHookEx", "hwnd", $hCtrlEvent_MHook[0])
		$hCtrlEvent_MHook[0] = 0
	EndIf
	
	If IsPtr($hCtrlEvent_CallBack) Then
		DllCallbackFree($hCtrlEvent_CallBack)
		$hCtrlEvent_CallBack = 0
	EndIf
EndFunc
